--
-- AdBlue Usablity
-- Specialization for using AdBlue
-- For FS15 ONLY
--
-- by Blacky_BPG
-- !!! DriveControl is needed !!!
--
-- Optional for almost real consumption: fuelAdjust V5.15.8 or higher
--
--
-- V 1.0		26.02.2016
--


--[[

	Vehicle XML Entry with default values
		!!! NOTE: delete the dashLight entry if you do not have one !!!
	============================================================================================================================
	<adBlueChange adBlueChangeCheckString="changeAdBlue" warnBlinking="true" dashLight="" adBlueCapacity="5.0" interval="3.0" />
	============================================================================================================================

	Description of values:
		adBlueChangeCheckString 			-- somewhat outdated because ... well just AdBlue not Oil
									-- this entry can be omitted
		warnBlinking					-- defines whether the warning lights are flashing or be displayed permanently on warning on
		dashLight						-- index to dashboard light if you have one, otherwise delete this entry, i mean really serious: DELETE IT
		adBlueCapacity					-- capacity of the AdBlue tank in liters
									-- it's an floating value, you can choose from 0.1 to whatever you want
		interval						-- how many times, in relation to the full tank capacity, AdBlue must be refueled
									-- it's an floating value, you can choose from 0.1 to whatever you want
									-- value 0.1 mean, for an 300 liters fuel tank, your AdBlue tank is emtpy after 30 liters fuel used
									-- value 2.5 mean, for an 300 liters fuel tank, your AdBlue tank is emtpy after 750 liters fuel used
]]


AdBlueChange = {};

function AdBlueChange.prerequisitesPresent(specializations)
    return true;
end;

function AdBlueChange:load(xmlFile)
	self.changeAdBlue = SpecializationUtil.callSpecializationsFunction("changeAdBlue");
	self.adBlueChange = {};
	self.adBlueWarnBlinkTimer = 0;
	self.adBlueWarnMode = 0;
	self.adBlueChange.checkString = Utils.getNoNil(getXMLString(xmlFile, "vehicle.adBlueChange#adBlueChangeCheckString"), "changeAdBlue");
	self.adBlueBlinking = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.adBlueChange#warnBlinking"), true);
	self.adBlueChange.adBlueInRange = nil;
	self.adBlueLight = nil;
	local adBlueLight = getXMLString(xmlFile, "vehicle.adBlueChange#dashLight");
	if adBlueLight ~= nil then
		adBlueLight = Utils.indexToObject(self.components, adBlueLight);
		if adBlueLight ~= nil then
			self.adBlueLight = adBlueLight;
		end;
	end;
	self.shouldChangeAdBlue = false;
	self.adBlueCapacity = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.adBlueChange#adBlueCapacity"), 5);
	self.adBlueChangeInterval = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.adBlueChange#interval"), 3);
	local fuelCapacity = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fuelCapacity"), 500);
	self.adBlueUsage = (fuelCapacity * self.adBlueChangeInterval) / self.adBlueCapacity;
	self.adBlueLevel = self.adBlueCapacity;
	self.lastFuelFillLevel = self.fuelFillLevel;
	self.adBlueTick = 0;
	self.checkAdBlueOverlay = Overlay:new("hudcheckAdBlueOverlay", Utils.getFilename("textures/adBlue_low.dds", self.baseDirectory), 0.834, 0.075, 0.0093, 0.0155);
end;

function AdBlueChange:delete()
end;

function AdBlueChange:readStream(streamId, connection)
	self.adBlueLevel = streamReadFloat32(streamId);
	self.adBlueCapacity = streamReadFloat32(streamId);
end;

function AdBlueChange:writeStream(streamId, connection)
	streamWriteFloat32(streamId, self.adBlueLevel);
	streamWriteFloat32(streamId, self.adBlueCapacity);
end;

function AdBlueChange:mouseEvent(posX, posY, isDown, isUp, button)
end;

function AdBlueChange:keyEvent(unicode, sym, modifier, isDown)
end;

function AdBlueChange:update(dt)
	if self.isMotorStarted then
		self.adBlueTick = self.adBlueTick + 1;
		if self.adBlueTick >= 30 then
			local adBlueLevel = self.adBlueLevel;
			if self.lastFuelFillLevel > self.fuelFillLevel then
				local adBlueUsed = (self.lastFuelFillLevel - self.fuelFillLevel) / self.adBlueUsage
				adBlueLevel = adBlueLevel - adBlueUsed;
			end;
			self.adBlueTick = 0;
			self:changeAdBlue(self, false, adBlueLevel, true);
			self.lastFuelFillLevel = self.fuelFillLevel;
		end;
	end;

	local adBlueChange = self.adBlueChange;
	local adBlueWarning = 100 / self.adBlueCapacity * self.adBlueLevel;
	local adBlueNeeded = self.adBlueCapacity - self.adBlueLevel;

	if self.shouldChangeAdBlue == false and adBlueWarning <= 0 then
		self.shouldChangeAdBlue = true;
	elseif adBlueWarning > 12 and self.shouldChangeAdBlue then
		self.shouldChangeAdBlue = false;
	end;
	if self.shouldChangeAdBlue then
		if self.isHired then
			self:dismiss();
		end;
		if self.isMotorStarted then
			self.isMotorStarted = false;
			if driveControl ~= nil then
				self.driveControl.manMotorStart.isMotorStarted = false;
				driveControl:motorStop(self,true);
			elseif self.stopMotor ~= nil then
				self:stopMotor(true);
			end;
		end;
	end;

	self.adBlueWarnMode = 0;
	if self.isClient then
		if self:getIsActive() then
			if adBlueChange.adBlueInRange ~= nil then
				if self:getIsActiveForInput() or self:getIsActiveForInput(false) then
					if InputBinding.hasEvent(InputBinding.ChangeAdBlue) then
						if adBlueWarning <= 12 and adBlueChange.adBlueInRange.adBlueCapacity >= adBlueNeeded and adBlueChange.adBlueInRange.changeAdBluePallet ~= nil then
							adBlueChange.adBlueInRange:changeAdBluePallet(self);
						end;
					end;
					if adBlueChange.adBlueInRange.adBlueCapacity <= (adBlueNeeded - 0.5) then
						g_currentMission:addWarning(string.format(g_i18n:getText("Out_Of_AdBlue"),tostring(math.floor(adBlueChange.adBlueInRange.adBlueCapacity * 10) / 10)) .. "\n" .. g_i18n:getText("Buy_More_AdBlue"));
					end;
				end;
			end;

			if adBlueWarning <= 12 then
				self.adBlueWarnBlinkTimer = self.adBlueWarnBlinkTimer + 1;
				if self.adBlueWarnBlinkTimer >= 48 then
					self.adBlueWarnBlinkTimer = 0;
				end;
			end;

			if self.adBlueBlinking then
				if adBlueWarning <= 5 then
					self.adBlueWarnMode = 2
				elseif adBlueWarning <= 8.5 then
					if self.adBlueWarnBlinkTimer ~= nil then
						if self.adBlueWarnBlinkTimer > 16 then
							self.adBlueWarnMode = 1;
						end;
					else
						self.adBlueWarnMode = 1;
					end;
				elseif adBlueWarning <= 12 then
					if self.adBlueWarnBlinkTimer ~= nil then
						if self.adBlueWarnBlinkTimer > 32 then
							self.adBlueWarnMode = 1;
						end;
					else
						self.adBlueWarnMode = 1;
					end;
				end;
			else
				if adBlueWarning <= 12 then
					self.adBlueWarnMode = 2;
				end;
			end;

			if self.adBlueLight ~= nil then
				if self.isMotorStarted then
					if self.adBlueWarnMode > 0 then
						setVisibility(self.adBlueLight, true);
					else
						setVisibility(self.adBlueLight, false);
					end;
				else
					setVisibility(self.adBlueLight, false);
				end;
				if self.driveControl.manMotorStart.startCnt > 0 and self.driveControl.manMotorStart.startCnt < 800 then
					setVisibility(self.adBlueLight, true)
				end;
			end;
		end;
	end;
end;

function AdBlueChange:draw()
	local adBlueChange = self.adBlueChange;
	local adBlueWarning = 100 / self.adBlueCapacity * self.adBlueLevel;
	local adBlueNeeded = self.adBlueCapacity - self.adBlueLevel;
	if adBlueChange.adBlueInRange ~= nil then
		if adBlueWarning <= 12 then
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("CHANGE_ADBLUE"),tostring(math.floor(adBlueNeeded * 10) / 10)), InputBinding.ChangeAdBlue);
		end;
	end;

	if self.isMotorStarted then
		if self.blinkWarning then
			if self.adBlueWarnMode > 0 then
				self.checkAdBlueOverlay:render();
			end;
		else
			if adBlueWarning <= 50 then
				self.checkAdBlueOverlay:render();
			end;
		end;
	else
		if self.driveControl ~= nil and self.driveControl.manMotorStart ~= nil and self.driveControl.manMotorStart.startCnt < 800 and self.driveControl.manMotorStart.startCnt > 0 then
			self.checkAdBlueOverlay:render();
		end;
	end;

	if self.shouldChangeAdBlue then
		g_currentMission:addWarning(string.format(g_i18n:getText("AdBlue_Change_Iminent"),tostring(math.floor(adBlueNeeded * 10) / 10)) .. "\n" .. g_i18n:getText("Change_The_AdBlue"));
	end;
end;

function AdBlueChange:changeAdBlue(adBlueChange, noEventSend, adBlueLevel, opt)
	local oldLevel = self.adBlueLevel;
	local adBlueNeeded = self.adBlueCapacity - self.adBlueLevel;
	if opt == nil or opt == false then
		if adBlueChange ~= nil and adBlueChange.adBlueCapacity ~= nil and adBlueChange ~= self then
			adBlueChange.adBlueCapacity = adBlueChange.adBlueCapacity - adBlueNeeded;
			self.shouldChangeAdBlue = false;
			self.adBlueLevel = self.adBlueLevel + adBlueNeeded;
		end;
	end;
	if adBlueLevel ~= nil then
		if adBlueLevel > self.adBlueCapacity then adBlueLevel = self.adBlueCapacity end;
		self.adBlueLevel = adBlueLevel;
	end;
	if oldLevel ~= self.adBlueLevel then
		AdBlueChangeOverrideEvent.sendEvent(self, self.adBlueLevel, noEventSend);
	end;
end;

function AdBlueChange:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	local adBlueLevel = getXMLFloat(xmlFile, key.."#adBlueLevel");
	if adBlueLevel ~= nil then
		self:changeAdBlue(self, true, adBlueLevel, true);
	end;
	return BaseMission.VEHICLE_LOAD_OK;
end;

function AdBlueChange:getSaveAttributesAndNodes(nodeIdent)
	local attributes = nil;
	attributes = 'adBlueLevel="'.. self.adBlueLevel .. '"';
	return attributes, nil;
end;


AdBlueChangeOverrideEvent = {};
AdBlueChangeOverrideEvent_mt = Class(AdBlueChangeOverrideEvent, Event);

InitEventClass(AdBlueChangeOverrideEvent, "AdBlueChangeOverrideEvent");

function AdBlueChangeOverrideEvent:emptyNew()
	local self = Event:new(AdBlueChangeOverrideEvent_mt);
	self.className="AdBlueChangeOverrideEvent";
	return self;
end;

function AdBlueChangeOverrideEvent:new(vehicle, adBlueLevel)
	local self = AdBlueChangeOverrideEvent:emptyNew()
	self.vehicle = vehicle;
	self.adBlueLevel = adBlueLevel;
	return self;
end;

function AdBlueChangeOverrideEvent:readStream(streamId, connection)
	local id = streamReadInt32(streamId);
	self.vehicle = networkGetObject(id);
	self.adBlueLevel = streamReadFloat32(streamId);
	self:run(connection);
end;

function AdBlueChangeOverrideEvent:writeStream(streamId, connection)
	streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteFloat32(streamId, self.adBlueLevel);
end;

function AdBlueChangeOverrideEvent:run(connection)
	if self.vehicle ~= nil then
		if not connection:getIsServer() then
			g_server:broadcastEvent(AdBlueChangeOverrideEvent:new(self.vehicle, self.adBlueLevel), nil, connection, self.vehicle);
		end;
		if self.vehicle.changeAdBlue ~= nil then
			self.vehicle:changeAdBlue(self.vehicle, true, self.adBlueLevel);
		end;
	end;
end;

function AdBlueChangeOverrideEvent.sendEvent(vehicle, adBlueLevel, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(AdBlueChangeOverrideEvent:new(vehicle, adBlueLevel), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(AdBlueChangeOverrideEvent:new(vehicle, adBlueLevel));
		end;
	end;
end;
