--
-- OilChange
-- Specialization for changing oil
--
-- original author	JoXXer / rewritten and expanded by Blacky_BPG
-- Version for FS15
-- works only with DriveControl
-- 

OilChange = {};

function OilChange.prerequisitesPresent(specializations)
    return true;
end;

function OilChange:load(xmlFile)

	self.blinkWarning = true;

	self.changeOil = SpecializationUtil.callSpecializationsFunction("changeOil");
	self.oilChange = {};
	self.oilWarnBlinkTimer = 0;
	self.oilWarnMode = 0;
	self.oilChange.checkString = Utils.getNoNil(getXMLString(xmlFile, "vehicle.oilChange#oilChangeCheckString"), "changeOil");
	self.oilChange.oilInRange = nil;
	self.oilLight = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.oilChange#dashLight"));
	self.shouldChangeOil = false;
	self.opTime = 0;
	self.nextOilChange = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.oilChange#firstOilChange"), 10);
	local intervalLoaded = getXMLInt(xmlFile, "vehicle.oilChange#interval");
	self.oilChangeInterval = Utils.getNoNil(intervalLoaded, 20);
	local neededOilLoaded = getXMLFloat(xmlFile, "vehicle.oilChange#neededOilPerChange");
	self.neededOilPerChange = Utils.getNoNil(neededOilLoaded, 7);
	if neededOilLoaded == nil or (neededOilLoaded ~= nil and neededOilLoaded < 1) then
		self.neededOilPerChange = 7;
		if SpecializationUtil.hasSpecialization(Sprayer, self.specializations) then
			self.neededOilPerChange = 10;
		elseif SpecializationUtil.hasSpecialization(Telehandler, self.specializations) or SpecializationUtil.hasSpecialization(Frontloader, self.specializations) then
			self.neededOilPerChange = 8.5;
		elseif SpecializationUtil.hasSpecialization(Shovel, self.specializations) and SpecializationUtil.hasSpecialization(ArticulatedAxis, self.specializations) then
			self.neededOilPerChange = 13.5;
		elseif SpecializationUtil.hasSpecialization(ArticulatedAxis, self.specializations) then
			self.neededOilPerChange = 7.5;
		elseif SpecializationUtil.hasSpecialization(Combine, self.specializations) then
			self.neededOilPerChange = 11.5;
		end;
	end;
	if intervalLoaded == nil or (intervalLoaded ~= nil and (intervalLoaded < 5 or intervalLoaded > 65)) then --angabe der std zeit in real
		self.oilChangeInterval = 20; -- zeit nach dem ersten wechsel /-weitere intervalle
		self.nextOilChange = 10; --zeit nach erstkauf in std
		if SpecializationUtil.hasSpecialization(Sprayer, self.specializations) then
			self.nextOilChange = self.nextOilChange * 0.85;
			self.oilChangeInterval = self.oilChangeInterval * 0.85;
		elseif SpecializationUtil.hasSpecialization(Telehandler, self.specializations) or SpecializationUtil.hasSpecialization(Frontloader, self.specializations) then
			self.nextOilChange = self.nextOilChange * 0.9;
			self.oilChangeInterval = self.oilChangeInterval * 0.9;
		elseif SpecializationUtil.hasSpecialization(Shovel, self.specializations) and SpecializationUtil.hasSpecialization(ArticulatedAxis, self.specializations) then
			self.nextOilChange = self.nextOilChange * 0.65;
			self.oilChangeInterval = self.oilChangeInterval * 0.65;
		elseif SpecializationUtil.hasSpecialization(ArticulatedAxis, self.specializations) then
			self.nextOilChange = self.nextOilChange * 0.95;
			self.oilChangeInterval = self.oilChangeInterval * 0.95;
		elseif SpecializationUtil.hasSpecialization(Combine, self.specializations) then
			self.nextOilChange = self.nextOilChange * 0.75;
			self.oilChangeInterval = self.oilChangeInterval * 0.75;
		end;
	end;
	self.checkOilOverlay = Overlay:new("hudcheckOilOverlay", Utils.getFilename("textures/oil_low.dds", self.baseDirectory), 0.834, 0.1446, 0.0093, 0.0155);
end;


function OilChange:delete()
end;

function OilChange:readStream(streamId, connection)
    self.shouldChangeOil = streamReadBool(streamId);
    self.nextOilChange = streamReadInt32(streamId);
end;

function OilChange:writeStream(streamId, connection)
    streamWriteBool(streamId, self.shouldChangeOil);
    streamWriteInt32(streamId, self.nextOilChange);
end;

function OilChange:mouseEvent(posX, posY, isDown, isUp, button)
end;

function OilChange:keyEvent(unicode, sym, modifier, isDown)
end;

function OilChange:update(dt)
	local oilChange = self.oilChange;
	local oilWarning = (self.nextOilChange - self.opTime) * 100;

	if self.shouldChangeOil == false and oilWarning <= 0 then
		self.shouldChangeOil = true;
	elseif oilWarning > 50 and self.shouldChangeOil then
		self.shouldChangeOil = false;
	end;
	if self.shouldChangeOil then
		if self.isHired then
			self:dismiss();
		end;
		if self.isMotorStarted then
			self.isMotorStarted = false;
			if driveControl ~= nil then
				self.driveControl.manMotorStart.isMotorStarted = false;
				driveControl:motorStop(self,true);
			elseif self.stopMotor ~= nil then
				self:stopMotor(true);
			end;
		end;
	end;

	if self:getIsActive() then
		if oilChange.oilInRange ~= nil then
			if self:getIsActiveForInput() then
				if InputBinding.hasEvent(InputBinding.ChangeOil) then
					if oilWarning <= 50 and oilChange.oilInRange.oilCapacity >= self.neededOilPerChange and oilChange.oilInRange.changeOilPallet ~= nil then
						oilChange.oilInRange:changeOilPallet(self);
					end;
				end;
				if oilChange.oilInRange.oilCapacity <= (self.neededOilPerChange - 0.5) then
					g_currentMission:addWarning(string.format(g_i18n:getText("Out_Of_Oil"),tostring(math.floor(oilChange.oilInRange.oilCapacity * 10) / 10)) .. "\n" .. g_i18n:getText("Buy_More_Oil"));
				end;
			end;
		end;

		if oilWarning <= 50 then
			self.oilWarnBlinkTimer = self.oilWarnBlinkTimer + 1;
			if self.oilWarnBlinkTimer >= 48 then
				self.oilWarnBlinkTimer = 0;
			end;
		end;

		self.oilWarnMode = 0;
		if oilWarning <= 10 then
			self.oilWarnMode = 2
		elseif oilWarning <= 25 then
			if self.oilWarnBlinkTimer ~= nil then
				if self.oilWarnBlinkTimer > 16 then
					self.oilWarnMode = 1;
				end;
			else
				self.oilWarnMode = 1;
			end;
		elseif oilWarning <= 50 then
			if self.oilWarnBlinkTimer ~= nil then
				if self.oilWarnBlinkTimer > 32 then
					self.oilWarnMode = 1;
				end;
			else
				self.oilWarnMode = 1;
			end;
		end;

		if self.isMotorStarted then
			if self.blinkWarning then
				if self.oilWarnMode > 0 then
					setVisibility(self.oilLight, true);
				else
					setVisibility(self.oilLight, false);
				end;
			else
				if oilWarning <= 50 then
					setVisibility(self.oilLight, true);
				else
					setVisibility(self.oilLight, false);
				end;
			end;
		else
			if self.driveControl ~= nil and self.driveControl.manMotorStart ~= nil and self.driveControl.manMotorStart.startCnt < 800 and self.driveControl.manMotorStart.startCnt > 0 then
				setVisibility(self.oilLight, true);
			else
				setVisibility(self.oilLight, false)
			end;
		end;
	end;
end;

function OilChange:updateTick(dt)
		if self.driveControl ~= nil and self.driveControl.hourCounter ~= nil and self.driveControl.hourCounter.hours ~= nil then
			self.opTime = self.driveControl.hourCounter.hours;
		end;
end;

function OilChange:draw()
	local oilChange = self.oilChange;
	local oilWarning = (self.nextOilChange - self.opTime) * 100;
	if oilChange.oilInRange ~= nil then
		if oilWarning <= 50 then
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("CHANGE_OIL"),tostring(math.floor(self.neededOilPerChange * 10) / 10)), InputBinding.ChangeOil);
		end;
	end;

	if self.isMotorStarted then
		if self.blinkWarning then
			if self.oilWarnMode > 0 then
				self.checkOilOverlay:render();
			end;
		else
			if oilWarning <= 50 then
				self.checkOilOverlay:render();
			end;
		end;
	else
		if self.driveControl ~= nil and self.driveControl.manMotorStart ~= nil and self.driveControl.manMotorStart.startCnt < 800 and self.driveControl.manMotorStart.startCnt > 0 then
			self.checkOilOverlay:render();
		end;
	end;
	if self.shouldChangeOil then
		g_currentMission:addWarning(string.format(g_i18n:getText("Oil_Change_Iminent"),tostring(math.floor(self.neededOilPerChange * 10) / 10)) .. "\n" .. g_i18n:getText("Change_The_Oil"));
	end;
end;

function OilChange:changeOil(oilChange, noEventSend, nextOilChange, opt)
	local oldChange = self.nextOilChange;
	if opt == nil or opt == false then
		if oilChange ~= nil and oilChange.oilCapacity ~= nil then
			oilChange.oilCapacity = oilChange.oilCapacity - self.neededOilPerChange;
			self.shouldChangeOil = false;
			self.nextOilChange = self.nextOilChange + self.oilChangeInterval;
		end;
	end;
	if nextOilChange ~= nil then
		self.nextOilChange = nextOilChange;
	end;
	if oldChange ~= self.nextOilChange then
		OilChangeOverrideEvent.sendEvent(self, self.nextOilChange, noEventSend);
	end;
end;

function OilChange:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if self.oilCapacity ~= nil then
		self.oilCapacity = self.oilCapacity + self.neededOilPerChange;
	end;
	local valueInt = getXMLFloat(xmlFile, key.."#nextOilChange");
	if valueInt ~= nil then
		self:changeOil(self, true, valueInt, true);
	else
		-- Fix for integration in a running game
		local opTime = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#workingHours"), 0);
		self:changeOil(self, true, opTime + self.oilChangeInterval, true);
	end;
	return BaseMission.VEHICLE_LOAD_OK;
end;

function OilChange:getSaveAttributesAndNodes(nodeIdent)
	local attributes = nil;
	attributes = 'nextOilChange="'.. self.nextOilChange .. '"';
	return attributes, nil;
end;

OilChangeOverrideEvent = {};
OilChangeOverrideEvent_mt = Class(OilChangeOverrideEvent, Event);

InitEventClass(OilChangeOverrideEvent, "OilChangeOverrideEvent");

function OilChangeOverrideEvent:emptyNew()
	local self = Event:new(OilChangeOverrideEvent_mt);
	self.className="OilChangeOverrideEvent";
	return self;
end;

function OilChangeOverrideEvent:new(vehicle, nextOilChange)
	local self = OilChangeOverrideEvent:emptyNew()
	self.vehicle = vehicle;
	self.nextOilChange = nextOilChange;
	return self;
end;

function OilChangeOverrideEvent:readStream(streamId, connection)
	local id = streamReadInt32(streamId);
	self.nextOilChange = streamReadFloat32(streamId);
	self.vehicle = networkGetObject(id);
	self:run(connection);
end;

function OilChangeOverrideEvent:writeStream(streamId, connection)
	streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteFloat32(streamId, self.nextOilChange);
end;

function OilChangeOverrideEvent:run(connection)
	if self.vehicle ~= nil then
		if not connection:getIsServer() then
			g_server:broadcastEvent(OilChangeOverrideEvent:new(self.vehicle, self.nextOilChange), nil, connection, self.vehicle);
		end;
		if self.vehicle.changeOil ~= nil then
			self.vehicle:changeOil(self.vehicle, true, self.nextOilChange);
		end;
	end;
end;

function OilChangeOverrideEvent.sendEvent(vehicle, nextOilChange, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(OilChangeOverrideEvent:new(vehicle, nextOilChange), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(OilChangeOverrideEvent:new(vehicle, nextOilChange));
		end;
	end;
end;
